#!/usr/bin/env ruby

require 'swak'

if ARGV.size < 2 || ARGV.first == '-h'
  puts "args <in.map/tped> <kind> [params...]"
  puts "  valid kind/params:"
  puts "    adj  <num_snps> : Adjacent SNPs in a window. num_snps indicates how many adjacent SNPs to use per window"
  puts "    olap <num_snps> <num_overlaps> : same as adj but overlap by <num_overlaps> snps between neighboring windows"
  puts "    shuffle <num_snps> <wins_per_region> : Windows form uniformly random partition of consective regions."
  puts "    sampled <num_snps> <region_size> <coverage> : Windows are sampled as a uniformly random subset of every region.  Regions overlap such that every SNP will be selected in one window in expectation.  Region size <region_size> adjacent SNPs."
  exit 1
end

fn_map = ARGV.shift
kind  = ARGV.shift

# Load markers
fin = File.new(fn_map)
markers = []
for line in fin
  line.chomp!
  f = line.split("\t", -1)
  marker_name = f[1]
  marker_pos = f[2].to_f_strict
  markers << [marker_pos, marker_name]
end

#################################
# Main switch
#################################
if kind == 'adj'
  raise "Not enough parameters" if ARGV.size < 1
  win_size = ARGV.shift.to_i_strict

  w = 0
  m = 0
  while m + win_size < markers.size
    win_start = markers[m][0]
    win_stop  = markers[m + win_size - 1][0]
    print [w, win_start, win_stop].join("\t")
    print "\t"

    puts (m...(m+win_size)).to_a.join(",")

    m += win_size
    w += 1
  end
elsif kind == 'olap'
  raise "Not enough parameters" if ARGV.size < 2
  win_size = ARGV.shift.to_i_strict
  olap_amount = ARGV.shift.to_i_strict

  w = 0
  m = 0
  while m + win_size < markers.size
    win_start = markers[m][0]
    win_stop  = markers[m + win_size - 1][0]
    print [w, win_start, win_stop].join("\t")
    print "\t"

    puts (m...(m+win_size)).to_a.join(",")

    m += win_size - olap_amount
    w += 1
  end
elsif kind == 'shuffle'
  raise "Not enough parameters" if ARGV.size < 2
  win_size = ARGV.shift.to_i_strict
  wins_in_shuffle = ARGV.shift.to_i_strict
  shuffle_size = win_size * wins_in_shuffle

  w = 0
  m = 0
  while m + shuffle_size < markers.size
    shuffled = (m...(m + shuffle_size)).to_a.shuffle

    for i in (0...wins_in_shuffle)
      win_ms = shuffled[i * win_size, win_size].sort

      win_start = markers[win_ms.first][0]
      win_stop  = markers[win_ms.last][0]
      print [w, win_start, win_stop].join("\t")
      print "\t"

      puts win_ms.join(",")

      w += 1
    end

    m += shuffle_size
  end
elsif kind == 'sampled'
  raise "Not enough parameters" if ARGV.size < 3
  win_size = ARGV.shift.to_i_strict
  region_size = ARGV.shift.to_i_strict
  coverage = ARGV.shift.to_f_strict

  w = 0
  m = 0

  # To simulate arbitrary coverage we will increment fractional indicies to indicate
  # the starting point of the next window
  mfloat = 0 # marker idx as a float
  floatinc = win_size.to_f / coverage.to_f # How much to increment

  while m < markers.size
    win_ms = []
    while win_ms.length < win_size
      x = -1
      x = m + rand(region_size) while x < 0 or x >= markers.length
      win_ms << x
    end
    win_ms.sort!

    win_start = markers[win_ms.first][0]
    win_stop  = markers[win_ms.last][0]

    print [w, win_start, win_stop].join("\t")
    print "\t"
    puts win_ms.join(",")

    w += 1
    mfloat += floatinc
    m = mfloat.to_i
  end
else
  raise "Unknown kind: #{kind}"
end

