#!/usr/bin/env python

import collections

do_haps = False
do_who = False

def print_usage():
    print('Usage: win-haplo-map.py [--haps] [--who] <fwins> <tped>')

def read_tped(tped_lines):
    tped = []
    for l in tped_lines:
        if do_haps and l[0] == '#': # skip header for haps
            continue
        row = l.split()
        chrom, label, pos_cm, pos_i = row[:4]
        if do_haps:
            nucs = row
        else:
            nucs = row[4:]
        tped.append(nucs)
    return tped

def print_map(fwins_lines, tped_lines):
    tped = read_tped(tped_lines)

    for fwin in fwins:
        w, _, _, snps = fwin.split()
        snps = [int(x) for x in snps.split(',')]
        haps = zip(*[tped[i] for i in snps])
        haps = [''.join(h) for h in haps]

        hap_to_idxs = collections.defaultdict(list)
        for i, h in enumerate(haps):
            hap_to_idxs[h].append(i)

        for h, idxs in hap_to_idxs.items():
            if do_who:
                print '\t'.join([w, h, str(len(idxs)), ','.join(str(i) for i in idxs)])
            else:
                print '\t'.join([w, h, str(len(idxs))])

if __name__ == '__main__':

    import sys

    args = sys.argv[1:]

    while len(args) >= 1 and args[0][0:2] == "--":
        if args[0] == "--haps":
            do_haps = True
            args = args[1:]
        elif args[0] == "--who":
            do_who = True
            args = args[1:]

    if len(args) < 2:
        print_usage()
        sys.exit(1)

    fwins_file, tped_file = args[:2]

    with open(fwins_file, 'r') as fwins:
        with open(tped_file, 'r') as tped:
            print_map(fwins, tped)
